<?php

namespace Drupal\image_srcset_formatter\Plugin\Field\FieldFormatter;

use Drupal\image\Plugin\Field\FieldFormatter\ImageFormatterBase;
use Drupal\image\Entity\ImageStyle;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Field\FieldItemListInterface;

/**
 * Plugin implementation of the 'image_srcset_formatter' formatter.
 *
 * @FieldFormatter(
 *   id = "image_srcset_formatter",
 *   label = @Translation("Image srcset"),
 *   field_types = {
 *     "image"
 *   }
 * )
 */
class ImageSrcsetFormatter extends ImageFormatterBase {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'srcset' => NULL,
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {

    $element = [];
    $breakpoints = [
      '320' => 'min-width: 320px',
      '480' => 'min-width: 480px',
      '768' => 'min-width: 768px',
      '992' => 'min-width: 992px',
      '1200' => 'min-width: 1200px',
      '1600' => 'min-width: 1600px',
      '1920' => 'min-width: 1920px',
    ];

    // Initialize the element variable.
    foreach ($breakpoints as $key => $breakpoint) {
      $element['srcset'][$key] = [
        '#type' => 'select',
        '#title' => $breakpoint,
        '#options' => image_style_options(),
        '#empty_option' => $this->t('None (original image)'),
        '#default_value' => $this->getSetting('srcset')[$key],
      ];
    }
    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {

    $summary = [];
    $srcset_settings = $this->getSetting('srcset');
    $has_style = FALSE;

    // Check if style has been set.
    if (!empty($srcset_settings)) {
      foreach ($srcset_settings as $style) {
        if (!empty($style)) {
          $has_style = TRUE;
        }
      }
    }
    if ($has_style == FALSE) {
      $summary[] = $this->t("Original image");
    }
    else {
      foreach (array_filter($srcset_settings) as $key => $srcset) {
        $summary[] .= format_string('min-width: ' . $key . 'px - (@srcset)', [
          '@srcset' => $srcset,
        ]);
      }
    }
    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $element = [];
    $srcset_settings = $this->getSetting('srcset');
    $src_size = [];
    $srcset_url = [];
    $img_alt = "";
    $img_title = "";
    $src_url = "";

    foreach ($items as $delta => $item) {
      // Get the original image for default.
      $image_uri = $item->entity->getFileUri();
      $src_url = file_create_url($image_uri);
      $img_alt = $item->getValue()['alt'];
      $img_title = $item->getValue()['title'];

      if (NULL !== ($srcset_settings) && !empty($srcset_settings)) {
        foreach ($srcset_settings as $size => $style) {
          // Check all the style that has been set.
          if (isset($style) && !empty($style)) {
            $styled_image = ImageStyle::load($style)->buildUrl($image_uri);
            $image_width = getimagesize($styled_image)[0];
            $srcset_url[$size] = $styled_image . ' ' . $image_width . 'w';
            $src_size[$size] = '(min-width: ' . $size . 'px) ' . $image_width . 'px';
          }
        }
      }

      // Arrange scrset and sizes.
      krsort($src_size, 1);
      krsort($srcset_url, 1);

      $element[$delta] = [
        '#theme' => 'image_srcset_formatter',
        '#src_url' => $src_url,
        '#srcset_url' => implode(", ", $srcset_url),
        '#sizes' => implode(", ", $src_size),
        '#img_alt' => $img_alt,
        '#img_title' => $img_title,
      ];
    }
    return $element;
  }

}
